from array import array
from datetime import datetime, timedelta
from functools import lru_cache
from struct import unpack
from typing import Dict, List

from numbers_parser.containers import ObjectStore
from numbers_parser.cell import xl_rowcol_to_cell, xl_col_to_name
from numbers_parser.exceptions import UnsupportedError
from numbers_parser.generated import TSTArchives_pb2 as TSTArchives
from numbers_parser.formula import TableFormulas


class NumbersModel:
    """
    Loads all objects from Numbers document and provides decoding
    methods for other classes in the module to abstract away the
    internal structures of Numbers document data structures.

    Not to be used in application code.
    """

    def __init__(self, filename):
        self.objects = ObjectStore(filename)
        self._row_headers = {}
        self._row_columns = {}
        self._table_strings = {}
        self._table_tiles = {}
        self._tables_in_sheet = {}
        self._table_base_ids = {}

    def find_refs(self, ref: str) -> list:
        return self.objects.find_refs(ref)

    def sheet_ids(self):
        return [o.identifier for o in self.objects[1].sheets]

    def sheet_name(self, sheet_id):
        return self.objects[sheet_id].name

    @lru_cache(maxsize=None)
    def table_ids(self, sheet_id):
        table_info_ids = self.find_refs("TableInfoArchive")
        return [
            self.objects[t_id].tableModel.identifier
            for t_id in table_info_ids
            if self.objects[t_id].super.parent.identifier == sheet_id
        ]

    @lru_cache(maxsize=None)
    def row_storage_map(self, table_id):
        # The base data store contains a reference to rowHeaders.buckets
        # which is an ordered list that matches the storage buffers, but
        # identifies which row a storage buffer belongs to (empty rows have
        # no storage buffers). Each bucket is:
        #
        #  {
        #      "hidingState": 0,
        #      "index": 0,
        #      "numberOfCells": 3,
        #      "size": 0.0
        #  },
        row_bucket_map = {i: None for i in range(self.objects[table_id].number_of_rows)}
        bds = self.objects[table_id].base_data_store
        buckets = self.objects[bds.rowHeaders.buckets[0].identifier].headers
        for i, bucket in enumerate(buckets):
            row_bucket_map[bucket.index] = i
        return row_bucket_map

    def number_of_rows(self, table_id):
        return self.objects[table_id].number_of_rows

    def number_of_columns(self, table_id):
        return self.objects[table_id].number_of_columns

    def table_name(self, table_id):
        return self.objects[table_id].table_name

    @lru_cache(maxsize=None)
    def table_tiles(self, table_id):
        bds = self.objects[table_id].base_data_store
        return [self.objects[t.tile.identifier] for t in bds.tiles.tiles]

    @lru_cache(maxsize=None)
    def table_string(self, table_id, key):
        bds = self.objects[table_id].base_data_store
        strings_id = bds.stringTable.identifier
        for x in self.objects[strings_id].entries:  # pragma: no branch
            if x.key == key:
                return x.string

    @lru_cache(maxsize=None)
    def owner_id_map(self):
        """ "
        Extracts the mapping table from Owner IDs to UUIDs. Returns a
        dictionary mapping the owner ID int to a 128-bit UUID.
        """
        # The TSCE.CalculationEngineArchive contains a list of mapping entries
        # in dependencyTracker.formulaOwnerDependencies from the root level
        # of the protobuf. Each mapping contains a 32-bit style UUID:
        #
        # "ownerIdMap": {
        #     "mapEntry": [
        #     {
        #         "internalOwnerId": 33,
        #         "ownerId": {
        #             "uuidW0": "0x8750e563, "uuidW1": "0x1e4bfcc0,
        #             "uuidW2": "0xc26dda92, "uuidW3": "0x3cb03f23
        #         }
        #     },
        #
        #
        ce_id = self.find_refs("CalculationEngineArchive")[0]
        calc_engine = self.objects[ce_id]
        owner_id_map = {}
        for e in calc_engine.dependency_tracker.owner_id_map.map_entry:
            owner_id_map[e.internal_owner_id] = uuid(e.owner_id)
        return owner_id_map

    @lru_cache(maxsize=None)
    def table_base_id(self, table_id: int) -> int:
        """ "Finds the UUID of a table"""
        # Look for a TSCE.FormulaOwnerDependenciesArchive objects with the following at the
        # root level of the protobuf:
        #
        #     "baseOwnerUid": {
        #         "lower": "0x0c4ebfb1d9676393",
        #         "upper": "0xf9ad9f35d33aba96"
        #     }
        #      "formulaOwnerUid": {
        #         "lower": "0x0c4ebfb1d96763b6",
        #         "upper": "0xf9ad9f35d33aba96"
        #     },
        #
        # The Table UUID is the TSCE.FormulaOwnerDependenciesArchive whose formulaOwnerUid
        # matches the UUID of the hauntedOwner of the Table:
        #
        #    "hauntedOwner": {
        #        "ownerUid": {
        #            "lower": "0x0c4ebfb1d96763b6",
        #            "upper": "0xf9ad9f35d33aba96"
        #        }
        #    }
        haunted_owner = uuid(self.objects[table_id].haunted_owner.owner_uid)
        formula_owner_ids = self.find_refs("FormulaOwnerDependenciesArchive")
        for dependency_id in formula_owner_ids:  # pragma: no branch
            obj = self.objects[dependency_id]
            if obj.HasField("base_owner_uid") and obj.HasField(
                "formula_owner_uid"
            ):  # pragma: no branch
                base_owner_uid = uuid(obj.base_owner_uid)
                formula_owner_uid = uuid(obj.formula_owner_uid)
                if formula_owner_uid == haunted_owner:
                    return base_owner_uid

    @lru_cache(maxsize=None)
    def formula_cell_ranges(self, table_id: int) -> list:
        """Exract all the formula cell ranges for the Table."""
        # The TSCE.CalculationEngineArchive contains formulaOwnerInfo records
        # inside the dependencyTracker. Each of these has a cellDependencies
        # dictionary and some contain cellRecords that contain formula references:
        #
        # "cellDependencies": {
        #     "cellRecord": [
        #         {
        #             "column": 0,
        #             "containsAFormula": true,
        #             "edges": { "packedEdgeWithoutOwner": [ 16777216 ] },
        #             "row": 1
        #         },
        #         {
        #             "column": 1,
        #             "containsAFormula": true,
        #             "edges": { "packedEdgeWithoutOwner": [16777216 ] },
        #             "row": 1
        #         }
        #     ]
        # }
        cell_records = []
        table_base_id = self.table_base_id(table_id)
        ce_id = self.find_refs("CalculationEngineArchive")[0]
        for finfo in self.objects[ce_id].dependency_tracker.formula_owner_info:
            if finfo.HasField("cell_dependencies"):
                formula_owner_id = uuid(finfo.formula_owner_id)
                if formula_owner_id == table_base_id:
                    for cell_record in finfo.cell_dependencies.cell_record:
                        if cell_record.contains_a_formula:
                            cell_records.append((cell_record.row, cell_record.column))
        return cell_records

    @lru_cache(maxsize=None)
    def merge_cell_ranges(self, table_id):
        """Exract all the merge cell ranges for the Table."""
        # Merge ranges are stored in a number of structures, but the simplest is
        # a TSCE.RangePrecedentsTileArchive which exists for each Table in the
        # Document. These archives contain a fromToRange list which has the merge
        # ranges associated with an Owner ID
        #
        # The Owner IDs need to be extracted from the Calculation Engine using
        # UUID matching (this seems fragile, but no other mechanism has been found)
        #
        #  "fromToRange": [
        #      {
        #          "fromCoord": {
        #          "column": 0,
        #          "row": 0
        #      },
        #      "refersToRect": {
        #          "origin": {
        #              "column": 0,
        #              "row": 0
        #          },
        #          "size": {
        #              "numColumns": 2
        #          }
        #      }
        #  ],
        #  "toOwnerId": 1
        #
        owner_id_map = self.owner_id_map()
        table_base_id = self.table_base_id(table_id)

        merge_cells = {}
        range_table_ids = self.find_refs("RangePrecedentsTileArchive")
        for range_id in range_table_ids:
            o = self.objects[range_id]
            to_owner_id = o.to_owner_id
            if owner_id_map[to_owner_id] == table_base_id:
                for from_to_range in o.from_to_range:
                    rect = from_to_range.refers_to_rect
                    row_start = rect.origin.row
                    row_end = row_start + rect.size.num_rows - 1
                    col_start = rect.origin.column
                    col_end = col_start + rect.size.num_columns - 1
                    for row_num in range(row_start, row_end + 1):
                        for col_num in range(col_start, col_end + 1):
                            merge_cells[(row_num, col_num)] = {
                                "merge_type": "ref",
                                "rect": (row_start, col_start, row_end, col_end),
                                "size": (rect.size.num_rows, rect.size.num_columns),
                            }
                    merge_cells[(row_start, col_start)]["merge_type"] = "source"
        return merge_cells

    @lru_cache(maxsize=None)
    def table_uuids_to_id(self, table_uuid):
        for t_id in self.find_refs("TableInfoArchive"):  # pragma: no branch
            table_model_id = self.objects[t_id].tableModel.identifier
            if table_uuid == self.table_base_id(table_model_id):
                return table_model_id

    def node_to_ref(self, row_num: int, col_num: int, node):
        table_name = None
        if node.HasField("AST_cross_table_reference_extra_info"):
            table_uuid = uuid(node.AST_cross_table_reference_extra_info.table_id)
            table_id = self.table_uuids_to_id(table_uuid)
            table_name = self.table_name(table_id)

        if node.HasField("AST_column") and not node.HasField("AST_row"):
            return node_to_col_ref(node, table_name, col_num)
        else:
            return node_to_row_col_ref(node, table_name, row_num, col_num)

    @lru_cache(maxsize=None)
    def formula_ast(self, table_id: int):
        bds = self.objects[table_id].base_data_store
        formula_table_id = bds.formula_table.identifier
        formula_table = self.objects[formula_table_id]
        formulas = {}
        for formula in formula_table.entries:
            formulas[formula.key] = formula.formula.AST_node_array.AST_node
        return formulas

    @lru_cache(maxsize=None)
    def storage_buffers_pre_bnc(self, table_id: int) -> List:
        row_infos = []
        for tile in self.table_tiles(table_id):
            row_infos += tile.rowInfos

        return [
            get_storage_buffers_for_row(
                r.cell_storage_buffer_pre_bnc,
                r.cell_offsets_pre_bnc,
                self.number_of_columns(table_id),
                r.has_wide_offsets,
            )
            for r in row_infos
        ]

    @lru_cache(maxsize=None)
    def storage_buffers(self, table_id: int) -> List:
        row_infos = []
        for tile in self.table_tiles(table_id):
            row_infos += tile.rowInfos

        return [
            get_storage_buffers_for_row(
                r.cell_storage_buffer,
                r.cell_offsets,
                self.number_of_columns(table_id),
                r.has_wide_offsets,
            )
            for r in row_infos
        ]

    @lru_cache(maxsize=None)
    def storage_buffer(self, table_id: int, row_num: int, col_num: int) -> bytes:
        row_offset = self.row_storage_map(table_id)[row_num]
        if row_offset is None:
            return None
        try:
            storage_buffers = self.storage_buffers(table_id)
            return storage_buffers[row_offset][col_num]
        except IndexError:
            return None

    @lru_cache(maxsize=None)
    def storage_buffer_pre_bnc(
        self, table_id: int, row_num: int, col_num: int
    ) -> bytes:
        row_offset = self.row_storage_map(table_id)[row_num]
        if row_offset is None:
            return None
        try:
            storage_buffers_pre_bnc = self.storage_buffers_pre_bnc(table_id)
            return storage_buffers_pre_bnc[row_offset][col_num]
        except IndexError:
            return None

    @lru_cache(maxsize=None)
    def table_formulas(self, table_id: int):
        return TableFormulas(self, table_id)

    @lru_cache(maxsize=None)
    def table_cell_decode(self, table_id: int, row_num: int, col_num: int) -> Dict:
        storage_buffer = self.storage_buffer(table_id, row_num, col_num)
        storage_buffer_pre_bnc = self.storage_buffer_pre_bnc(table_id, row_num, col_num)

        if storage_buffer is None:
            return None

        cell_type = storage_buffer[1]
        cell_value = None

        if cell_type == TSTArchives.numberCellType or cell_type == 10:
            if storage_buffer_pre_bnc is None:
                cell_value = 0.0
            else:
                cell_value = unpack("<d", storage_buffer_pre_bnc[-12:-4])[0]
            cell_type = TSTArchives.numberCellType
        elif cell_type == TSTArchives.textCellType:
            string_key = unpack("<i", storage_buffer[12:16])[0]
            cell_value = self.table_string(table_id, string_key)
        elif cell_type == TSTArchives.dateCellType:
            if storage_buffer_pre_bnc is None:
                cell_value = datetime(2001, 1, 1)
            else:
                seconds = unpack("<d", storage_buffer_pre_bnc[-12:-4])[0]
                cell_value = datetime(2001, 1, 1) + timedelta(seconds=seconds)
        elif cell_type == TSTArchives.boolCellType:
            cell_value = unpack("<d", storage_buffer[12:20])[0] > 0.0
        elif cell_type == TSTArchives.durationCellType:
            cell_value = unpack("<d", storage_buffer[12:20])[0]
        elif cell_type == TSTArchives.currencyCellValueType:
            cell_value = unpack("<d", storage_buffer_pre_bnc[-12:-4])[0]

        return {"type": cell_type, "value": cell_value}

    @lru_cache(maxsize=None)
    def table_cell_formula_decode(
        self, table_id: int, row_num: int, col_num: int, cell_type: int
    ):
        storage_buffer = self.storage_buffer(table_id, row_num, col_num)
        if not self.table_formulas(table_id).is_formula(row_num, col_num):
            return None

        # TODO: Why is this required? Which cell types are affected?
        mod_offset = unpack("<h", storage_buffer[6:8])[0] == 1

        if cell_type == TSTArchives.formulaErrorCellType:
            formula_key = unpack("<h", storage_buffer[-4:-2])[0]
        elif (
            cell_type == TSTArchives.textCellType
            or cell_type == TSTArchives.durationCellType
        ) and mod_offset:
            formula_key = unpack("<h", storage_buffer[-16:-14])[0]
        else:
            formula_key = unpack("<h", storage_buffer[-12:-10])[0]

        return formula_key


def node_to_col_ref(node: object, table_name: str, col_num: int) -> str:
    if node.AST_column.absolute:
        col = node.AST_column.column
    else:
        col = col_num + node.AST_column.column

    col_name = xl_col_to_name(col, node.AST_column.absolute)
    if table_name is not None:
        return f"{table_name}::{col_name}"
    else:
        return col_name


def node_to_row_col_ref(
    node: object, table_name: str, row_num: int, col_num: int
) -> str:
    if node.AST_row.absolute:
        row = node.AST_row.row
    else:
        row = row_num + node.AST_row.row
    if node.AST_column.absolute:
        col = node.AST_column.column
    else:
        col = col_num + node.AST_column.column

    ref = xl_rowcol_to_cell(
        row,
        col,
        row_abs=node.AST_row.absolute,
        col_abs=node.AST_column.absolute,
    )
    if table_name is not None:
        return f"{table_name}::{ref}"
    else:
        return ref


def uuid(ref: dict) -> int:
    """
    Extract storage buffers for each cell in a table row

    Args:
        ref: Google protobuf containing either four 32-bit IDs or two 64-bit IDs

    Returns:
        uuid: 128-bit UUID

    Raises:
        UnsupportedError: object does not include expected UUID fields
    """
    try:
        if hasattr(ref, "upper"):
            uuid = ref.upper << 64 | ref.lower
        else:
            uuid = (
                (ref.uuid_w3 << 96)
                | (ref.uuid_w2 << 64)
                | (ref.uuid_w1 << 32)
                | ref.uuid_w0
            )
    except AttributeError:
        raise UnsupportedError(f"Unsupported UUID structure: {ref}")  # pragma: no cover

    return uuid


def get_storage_buffers_for_row(
    storage_buffer: bytes, offsets: list, num_cols: int, has_wide_offsets: bool
) -> List[bytes]:
    """
    Extract storage buffers for each cell in a table row

    Args:
        storage_buffer:  cell_storage_buffer or cell_storage_buffer for a table row
        offsets: 16-bit cell offsets for a table row
        num_cols: number of columns in a table row
        has_wide_offsets: use 4-byte offsets rather than 1-byte offset

    Returns:
         data: list of bytes for each cell in a row, or None if empty
    """
    offsets = array("h", offsets).tolist()
    if has_wide_offsets:
        offsets = [o * 4 for o in offsets]

    data = []
    for col_num in range(num_cols):
        if col_num >= len(offsets):
            break

        start = offsets[col_num]
        if start < 0:
            data.append(None)
            continue

        if col_num == (len(offsets) - 1):
            end = len(storage_buffer)
        else:
            # Get next offset past current one that is not -1
            # https://stackoverflow.com/questions/19502378/
            idx = next(
                (i for i, x in enumerate(offsets[col_num + 1 :]) if x >= 0), None
            )
            if idx is None:
                end = len(storage_buffer)
            else:
                end = offsets[col_num + idx + 1]
        data.append(storage_buffer[start:end])

    return data



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: TNArchives_sos.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import numbers_parser.generated.TSDArchives_sos_pb2 as TSDArchives__sos__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='TNArchives_sos.proto',
  package='TNSOS',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x14TNArchives_sos.proto\x12\x05TNSOS\x1a\x15TSDArchives_sos.proto\"c\n\"SheetStylePropertyChangeSetArchive\x12%\n\x04\x66ill\x18\x01 \x01(\x0b\x32\x17.TSDSOS.SpecFillArchive\x12\x16\n\x0e\x66ill_undefined\x18\x02 \x01(\x08'
  ,
  dependencies=[TSDArchives__sos__pb2.DESCRIPTOR,])




_SHEETSTYLEPROPERTYCHANGESETARCHIVE = _descriptor.Descriptor(
  name='SheetStylePropertyChangeSetArchive',
  full_name='TNSOS.SheetStylePropertyChangeSetArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fill', full_name='TNSOS.SheetStylePropertyChangeSetArchive.fill', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='fill_undefined', full_name='TNSOS.SheetStylePropertyChangeSetArchive.fill_undefined', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=54,
  serialized_end=153,
)

_SHEETSTYLEPROPERTYCHANGESETARCHIVE.fields_by_name['fill'].message_type = TSDArchives__sos__pb2._SPECFILLARCHIVE
DESCRIPTOR.message_types_by_name['SheetStylePropertyChangeSetArchive'] = _SHEETSTYLEPROPERTYCHANGESETARCHIVE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SheetStylePropertyChangeSetArchive = _reflection.GeneratedProtocolMessageType('SheetStylePropertyChangeSetArchive', (_message.Message,), {
  'DESCRIPTOR' : _SHEETSTYLEPROPERTYCHANGESETARCHIVE,
  '__module__' : 'TNArchives_sos_pb2'
  # @@protoc_insertion_point(class_scope:TNSOS.SheetStylePropertyChangeSetArchive)
  })
_sym_db.RegisterMessage(SheetStylePropertyChangeSetArchive)


# @@protoc_insertion_point(module_scope)



__doc__ = """Parse and extract data from Apple Numbers spreadsheets

numbers_parser is a Python module for parsing Apple Numbers .numbers files.
It supports Numbers files generated by Numbers version 10.3 and 11.0
(current as of May 2021).

It supports and is tested against Python versions from 3.5 onwards. It is
not compatible with earlier versions of Python.

Currently supported features of Numbers files are:

- Multiple sheets per document
- Multiple tables per sheet
- Text, numeric, date, currency, duration, percentage cell types

Formulas have very limited support and rely wholly on Numbers saving values
in cells as part of the saved document, which is not always guaranteed. When
a formula value is not present, the value *FORMULA* is returned. Any formula
that results in a Numbers error returns a value *ERROR*.
"""

import os
import plistlib
import warnings

from numbers_parser.document import Document  # NOQA
from numbers_parser.cell import *  # NOQA
from numbers_parser._version import __version__  # NOQA

_DEFAULT_NUMBERS_INSTALL_PATH = "/Applications/Numbers.app"
_VERSION_PLIST_PATH = "Contents/version.plist"
_SUPPORTED_NUMBERS_VERSIONS = [
    "10.3",
    "11.0",
    "11.1",
    "11.2",
]

# Don't print the source line
formatwarning_old = warnings.formatwarning
warnings.formatwarning = lambda message, category, filename, lineno, line=None: formatwarning_old(  # pragma: no cover
    message, category, filename, lineno, line=""
)


def _get_version():
    return __version__


def _check_installed_numbers_version():
    try:
        fp = open(
            os.path.join(_DEFAULT_NUMBERS_INSTALL_PATH, _VERSION_PLIST_PATH), "rb"
        )
    except IOError:  # pragma: no cover
        return None
    version_dict = plistlib.load(fp)
    installed_version = version_dict["CFBundleShortVersionString"]
    if installed_version not in _SUPPORTED_NUMBERS_VERSIONS:
        warnings.warn(
            f"Numbers version {installed_version} not tested with this version"
        )
    fp.close()


_check_installed_numbers_version()



from functools import lru_cache
from typing import Union, Generator

from numbers_parser.containers import ItemsList
from numbers_parser.model import NumbersModel

from numbers_parser.cell import (
    Cell,
    MergedCell,
    xl_cell_to_rowcol,
    xl_range,
)


class Document:
    def __init__(self, filename):
        self._model = NumbersModel(filename)

    def sheets(self):
        if not hasattr(self, "_sheets"):
            refs = self._model.sheet_ids()
            self._sheets = ItemsList(self._model, refs, Sheet)
        return self._sheets


class Sheet:
    def __init__(self, model, sheet_id):
        self._sheet_id = sheet_id
        self._model = model

    def tables(self):
        if not hasattr(self, "_tables"):
            refs = self._model.table_ids(self._sheet_id)
            self._tables = ItemsList(self._model, refs, Table)
        return self._tables

    @property
    def name(self):
        return self._model.sheet_name(self._sheet_id)


class Table:
    def __init__(self, model, table_id):
        super(Table, self).__init__()
        self._model = model
        self._table_id = table_id

    @property
    def name(self):
        return self._model.table_name(self._table_id)

    @lru_cache(maxsize=None)
    def rows(self, values_only: bool = False) -> list:
        """
        Return all rows of cells for the Table.

        Args:
            values_only: if True, return cell values instead of Cell objects

        Returns:
            rows: list of rows; each row is a list of Cell objects
        """
        row_cells = []
        if values_only:
            for row_num in range(self.num_rows):
                row_cells.append(
                    [
                        self.cell(row_num, col_num).value
                        for col_num in range(self.num_cols)
                    ]
                )
        else:
            for row_num in range(self.num_rows):
                row_cells.append(
                    [self.cell(row_num, col_num) for col_num in range(self.num_cols)]
                )
        return row_cells

    @property
    @lru_cache(maxsize=None)
    def merge_ranges(self) -> list:
        merge_cells = self._model.merge_cell_ranges(self._table_id)
        ranges = [xl_range(*r["rect"]) for r in merge_cells.values()]
        return sorted(set(list(ranges)))

    @property
    def num_rows(self) -> int:
        """Number of rows in the table"""
        return self._model.number_of_rows(self._table_id)

    @property
    def num_cols(self) -> int:
        """Number of columns in the table"""
        return self._model.number_of_columns(self._table_id)

    def cell(self, *args) -> Union[Cell, MergedCell]:
        if type(args[0]) == str:
            (row_num, col_num) = xl_cell_to_rowcol(args[0])
        elif len(args) != 2:
            raise IndexError("invalid cell reference " + str(args))
        else:
            (row_num, col_num) = args

        if row_num >= self.num_rows or row_num < 0:
            raise IndexError(f"row {row_num} out of range")
        if col_num >= self.num_cols or col_num < 0:
            raise IndexError(f"coumn {col_num} out of range")

        return Cell.factory(self._model, self._table_id, row_num, col_num)

    def iter_rows(
        self,
        min_row: int = None,
        max_row: int = None,
        min_col: int = None,
        max_col: int = None,
        values_only: bool = False,
    ) -> Generator[tuple, None, None]:
        """
        Produces cells from a table, by row. Specify the iteration range using
        the indexes of the rows and columns.

        Args:
            min_row: smallest row index (zero indexed)
            max_row: largest row (zero indexed)
            min_col: smallest row index (zero indexed)
            max_col: largest row (zero indexed)
            values_only: return cell values rather than Cell objects

        Returns:
            generator: tuple of cells

        Raises:
            IndexError: row or column values are out of range for the table
        """
        min_row = min_row or 0
        max_row = max_row or self.num_rows - 1
        min_col = min_col or 0
        max_col = max_col or self.num_cols - 1

        if min_row < 0:
            raise IndexError(f"row {min_row} out of range")
        if max_row > self.num_rows:
            raise IndexError(f"row {max_row} out of range")
        if min_col < 0:
            raise IndexError(f"column {min_col} out of range")
        if max_col > self.num_cols:
            raise IndexError(f"column {max_col} out of range")

        rows = self.rows()
        for row_num in range(min_row, max_row + 1):
            if values_only:
                yield tuple(
                    cell.value or None for cell in rows[row_num][min_col : max_col + 1]
                )
            else:
                yield tuple(rows[row_num][min_col : max_col + 1])

    def iter_cols(
        self,
        min_col: int = None,
        max_col: int = None,
        min_row: int = None,
        max_row: int = None,
        values_only: bool = False,
    ) -> Generator[tuple, None, None]:
        """
        Produces cells from a table, by column. Specify the iteration range using
        the indexes of the rows and columns.

        Args:
            min_col: smallest row index (zero indexed)
            max_col: largest row (zero indexed)
            min_row: smallest row index (zero indexed)
            max_row: largest row (zero indexed)
            values_only: return cell values rather than Cell objects

        Returns:
            generator: tuple of cells

        Raises:
            IndexError: row or column values are out of range for the table
        """
        min_row = min_row or 0
        max_row = max_row or self.num_rows - 1
        min_col = min_col or 0
        max_col = max_col or self.num_cols - 1

        if min_row < 0:
            raise IndexError(f"row {min_row} out of range")
        if max_row > self.num_rows:
            raise IndexError(f"row {max_row} out of range")
        if min_col < 0:
            raise IndexError(f"column {min_col} out of range")
        if max_col > self.num_cols:
            raise IndexError(f"column {max_col} out of range")

        rows = self.rows()
        for col_num in range(min_col, max_col + 1):
            if values_only:
                yield tuple(row[col_num].value for row in rows[min_row : max_row + 1])
            else:
                yield tuple(row[col_num] for row in rows[min_row : max_row + 1])



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: TSKArchives_sos.proto
"""Generated protocol buffer code."""
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import numbers_parser.generated.TSKArchives_pb2 as TSKArchives__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='TSKArchives_sos.proto',
  package='TSKSOS',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x15TSKArchives_sos.proto\x12\x06TSKSOS\x1a\x11TSKArchives.proto\"\x8c\x01\n\x1e\x46ixCorruptedDataCommandArchive\x12\"\n\x05super\x18\x01 \x02(\x0b\x32\x13.TSK.CommandArchive\x12\x1d\n\x15\x63orrupted_digest_list\x18\x02 \x03(\t\x12\'\n\x1f\x63orrupted_digest_list_undefined\x18\x03 \x01(\x08'
  ,
  dependencies=[TSKArchives__pb2.DESCRIPTOR,])




_FIXCORRUPTEDDATACOMMANDARCHIVE = _descriptor.Descriptor(
  name='FixCorruptedDataCommandArchive',
  full_name='TSKSOS.FixCorruptedDataCommandArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TSKSOS.FixCorruptedDataCommandArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='corrupted_digest_list', full_name='TSKSOS.FixCorruptedDataCommandArchive.corrupted_digest_list', index=1,
      number=2, type=9, cpp_type=9, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='corrupted_digest_list_undefined', full_name='TSKSOS.FixCorruptedDataCommandArchive.corrupted_digest_list_undefined', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=53,
  serialized_end=193,
)

_FIXCORRUPTEDDATACOMMANDARCHIVE.fields_by_name['super'].message_type = TSKArchives__pb2._COMMANDARCHIVE
DESCRIPTOR.message_types_by_name['FixCorruptedDataCommandArchive'] = _FIXCORRUPTEDDATACOMMANDARCHIVE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

FixCorruptedDataCommandArchive = _reflection.GeneratedProtocolMessageType('FixCorruptedDataCommandArchive', (_message.Message,), {
  'DESCRIPTOR' : _FIXCORRUPTEDDATACOMMANDARCHIVE,
  '__module__' : 'TSKArchives_sos_pb2'
  # @@protoc_insertion_point(class_scope:TSKSOS.FixCorruptedDataCommandArchive)
  })
_sym_db.RegisterMessage(FixCorruptedDataCommandArchive)


# @@protoc_insertion_point(module_scope)



from zipfile import ZipFile, BadZipFile
from numbers_parser.iwafile import IWAFile
from numbers_parser.exceptions import FileError, FileFormatError
from io import BytesIO

import os


def read_numbers_file(path, handler, store_objects=True):
    if os.path.isdir(path):
        if os.path.isfile(os.path.join(path, "Index.zip")):
            _get_objects_from_zip_file(os.path.join(path, "Index.zip"), handler, store_objects)
        else:
            for filename in os.listdir(path):
                filepath = os.path.join(path, filename)
                if os.path.isdir(filepath):
                    read_numbers_file(filepath, handler, store_objects)
                else:
                    f = open(filepath, "rb")
                    if filename.endswith(".iwa"):
                        contents = f.read()
                        _extract_iwa_archives(contents, filepath, handler, store_objects)
                    elif not store_objects:
                        contents = f.read()
                        handler(contents, os.path.join(path, filename))
    else:
        try:
            zipf = ZipFile(path)
        except BadZipFile as e:  # pragma: no cover
            raise FileError(f"{path}: " + str(e))

        if "Index.zip" in zipf.namelist():
            index_data = BytesIO(zipf.read("Index.zip"))
            _get_objects_from_zip_stream(ZipFile(index_data), handler, store_objects)
        else:
            _get_objects_from_zip_stream(zipf, handler, store_objects)


def _get_objects_from_zip_file(path, handler, store_objects=True):
    try:
        zipf = ZipFile(path)
    except BadZipFile as e:  # pragma: no cover
        raise FileError(f"{path}: " + str(e))

    _get_objects_from_zip_stream(zipf, handler, store_objects)


def _get_objects_from_zip_stream(zipf, handler, store_objects):
    for filename in zipf.namelist():
        if filename.endswith(".iwa"):
            contents = zipf.read(filename)
            _extract_iwa_archives(contents, filename, handler, store_objects)
        elif not store_objects:
            handler(zipf.read(filename), filename)


def _extract_iwa_archives(contents, filename, handler, store_objects):
    # TODO: LZFSE compressed according to /usr/bin/file
    if store_objects and "OperationStorage" in filename:
        return

    try:
        iwaf = IWAFile.from_buffer(contents, filename)
    except Exception as e:  # pragma: no cover
        raise FileFormatError(f"{filename}: invalid IWA file {filename}") from e

    if store_objects:
        if len(iwaf.chunks) != 1:
            raise FileFormatError(
                f"{filename}: chunk count != 1 in {filename}"
            )  # pragma: no cover
        for archive in iwaf.chunks[0].archives:
            if len(archive.objects) == 0:
                raise FileFormatError(f"{filename}: no objects in {filename}")  # pragma: no cover

            identifier = archive.header.identifier
            # TODO: what should we do for len(archive.objects) > 1?
            handler(identifier, archive.objects[0])
    else:
        handler(iwaf, filename)



# vi: ft=python

import argparse
import logging
import sys

from numbers_parser import Document, _get_version
from numbers_parser.exceptions import FileFormatError


def command_line_parser():
    parser = argparse.ArgumentParser(
        description="Export data from Apple Numbers spreadsheet tables"
    )
    commands = parser.add_mutually_exclusive_group()
    commands.add_argument(
        "-T",
        "--list-tables",
        action="store_true",
        help="List the names of tables and exit",
    )
    commands.add_argument(
        "-S",
        "--list-sheets",
        action="store_true",
        help="List the names of sheets and exit",
    )
    commands.add_argument(
        "-b",
        "--brief",
        action="store_true",
        default=False,
        help="Don't prefix data rows with name of sheet/table (default: false)",
    )
    parser.add_argument("-V", "--version", action="store_true")
    parser.add_argument("--debug", action="store_true", help="Enable debug output")
    parser.add_argument(
        "--formulas",
        action="store_true",
        help="Dump formulas instead of formula results",
    )
    parser.add_argument(
        "-s", "--sheet", action="append", help="Names of sheet(s) to include in export"
    )
    parser.add_argument(
        "-t", "--table", action="append", help="Names of table(s) to include in export"
    )
    parser.add_argument("document", nargs="*", help="Document(s) to export")
    return parser


def print_sheet_names(filename):
    for sheet in Document(filename).sheets():
        print(f"{filename}: {sheet.name}")


def print_table_names(filename):
    for sheet in Document(filename).sheets():
        for table in sheet.tables():
            print(f"{filename}: {sheet.name}: {table.name}")


def cell_as_string(args, cell):
    if cell is None:
        return ""
    elif args.formulas and cell.formula is not None:
        return cell.formula
    elif cell.value is None:
        return ""
    else:
        return str(cell.value)


def print_table(args, filename):
    for sheet in Document(filename).sheets():
        if args.sheet is not None and sheet.name not in args.sheet:
            continue
        for table in sheet.tables():
            if args.table is not None and table.name not in args.table:
                continue
            for row in table.rows():
                cells = [cell_as_string(args, cell) for cell in row]
                cells_str = ",".join(cells)
                if args.brief:
                    print(cells_str)
                else:
                    print(f"{filename}: {sheet.name}: {table.name}:", cells_str)


def main():
    parser = command_line_parser()
    args = parser.parse_args()

    if args.debug:
        logging.basicConfig(level=logging.DEBUG)

    if args.version:
        print(_get_version())
    elif len(args.document) == 0:
        parser.print_help()
    else:
        for filename in args.document:
            try:
                if args.list_sheets:
                    print_sheet_names(filename)
                elif args.list_tables:
                    print_table_names(filename)
                else:
                    print_table(args, filename)
            except FileFormatError as e:
                print(f"{filename}:", str(e), file=sys.stderr)
                sys.exit(1)


if __name__ == "__main__":
    # execute only if run as a script
    main()



# vi: ft=python

import argparse
import os
import json
import re
import sys

from numbers_parser.unpack import read_numbers_file
from numbers_parser import _get_version
from numbers_parser.iwafile import IWAFile
from numbers_parser.exceptions import FileFormatError


def ensure_directory_exists(prefix, path):
    """Ensure that a path's directory exists."""
    parts = os.path.split(path)
    try:
        os.makedirs(os.path.join(*([prefix] + list(parts[:-1]))))
    except OSError:
        pass


def convert_uuids_to_hex(obj):
    if isinstance(obj, dict):
        for k, v in obj.items():
            if isinstance(v, dict) or isinstance(v, list):
                convert_uuids_to_hex(v)
            elif k == "lower" or k == "upper":
                obj[k] = "0x{0:0{1}X}".format(int(v), 16)
            elif k in ["uuidW0", "uuidW1", "uuidW2", "uuidW3"]:
                obj[k] = "0x{0:0{1}X}".format(v, 8)
    elif isinstance(obj, list):
        for v in obj:
            if isinstance(v, dict) or isinstance(v, list):
                convert_uuids_to_hex(v)


def process_file(contents, filename, output_dir, hex_uuids):
    filename = re.sub(r".*\.numbers/", "", filename)
    ensure_directory_exists(output_dir, filename)
    target_path = os.path.join(output_dir, filename)
    if isinstance(contents, IWAFile):
        target_path = target_path.replace(".iwa", "")
        target_path += ".txt"
        with open(target_path, "w") as out:
            data = contents.to_dict()
            if hex_uuids:
                convert_uuids_to_hex(data)
            print(json.dumps(data, sort_keys=True, indent=4), file=out)
    else:
        with open(target_path, "wb") as out:
            out.write(contents)


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("document", help="Apple Numbers file(s)", nargs="*")
    parser.add_argument("-V", "--version", action="store_true")
    parser.add_argument("--hex-uuids", action="store_true", help="print UUIDs as hex")
    parser.add_argument("--output", "-o", help="directory name to unpack into")
    args = parser.parse_args()
    if args.version:
        print(_get_version())
    elif args.output is not None and len(args.document) > 1:
        print(
            "unpack-numbers: error: output directory only valid with a single document",
            file=sys.stderr,
        )
        sys.exit(1)
    elif len(args.document) == 0:
        parser.print_help()
    else:
        for document in args.document:
            output_dir = args.output or document.replace(".numbers", "")
            try:
                read_numbers_file(
                    document,
                    handler=lambda contents, filename: process_file(
                        contents, filename, output_dir, args.hex_uuids
                    ),
                    store_objects=False,
                )
            except FileFormatError as e:
                print(f"{document}:", str(e), file=sys.stderr)
                sys.exit(1)


if __name__ == "__main__":
    # execute only if run as a script
    main()



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: TNArchives.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


import numbers_parser.generated.TSPMessages_pb2 as TSPMessages__pb2
import numbers_parser.generated.TSKArchives_pb2 as TSKArchives__pb2
import numbers_parser.generated.TSCHArchives_pb2 as TSCHArchives__pb2
import numbers_parser.generated.TSCEArchives_pb2 as TSCEArchives__pb2
import numbers_parser.generated.TSSArchives_pb2 as TSSArchives__pb2
import numbers_parser.generated.TSDArchives_pb2 as TSDArchives__pb2
import numbers_parser.generated.TSWPArchives_pb2 as TSWPArchives__pb2
import numbers_parser.generated.TSAArchives_pb2 as TSAArchives__pb2
import numbers_parser.generated.TSTArchives_pb2 as TSTArchives__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='TNArchives.proto',
  package='TN',
  syntax='proto2',
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n\x10TNArchives.proto\x12\x02TN\x1a\x11TSPMessages.proto\x1a\x11TSKArchives.proto\x1a\x12TSCHArchives.proto\x1a\x12TSCEArchives.proto\x1a\x11TSSArchives.proto\x1a\x11TSDArchives.proto\x1a\x12TSWPArchives.proto\x1a\x11TSAArchives.proto\x1a\x11TSTArchives.proto\"\xcd\x03\n\x13SheetUIStateArchive\x12\x12\n\nview_scale\x18\x01 \x02(\x02\x12#\n\x0fscroll_position\x18\x02 \x02(\x0b\x32\n.TSP.Point\x12\x1b\n\x13previous_view_scale\x18\x03 \x01(\x02\x12#\n\x1bscroll_position_is_unscaled\x18\x04 \x01(\x08\x12,\n\x18previous_scroll_position\x18\x05 \x01(\x0b\x32\n.TSP.Point\x12\x1d\n\x15scroll_position_valid\x18\x06 \x01(\x08\x12&\n\x1eprevious_scroll_position_valid\x18\x07 \x01(\x08\x12\x1f\n\x0cvisible_size\x18\x08 \x01(\x0b\x32\t.TSP.Size\x12(\n\x15previous_visible_size\x18\t \x01(\x0b\x32\t.TSP.Size\x12\x14\n\x0c\x64\x65vice_idiom\x18\n \x01(\r\x12\x31\n\x0eselection_path\x18\x0b \x01(\x0b\x32\x19.TSK.SelectionPathArchive\x12\x32\n\x1aselection_path_transformer\x18\x0c \x01(\x0b\x32\x0e.TSP.Reference\"s\n\"SheetUIStateDictionaryEntryArchive\x12\x1d\n\x05sheet\x18\x01 \x02(\x0b\x32\x0e.TSP.Reference\x12.\n\rsheet_uistate\x18\x02 \x02(\x0b\x32\x17.TN.SheetUIStateArchive\"r\n!UUIDSheetUIStateDictionaryArchive\x12\x1d\n\nsheet_uuid\x18\x01 \x02(\x0b\x32\t.TSP.UUID\x12.\n\rsheet_uistate\x18\x02 \x02(\x0b\x32\x17.TN.SheetUIStateArchive\"\xd6\x0b\n\x0eUIStateArchive\x12\x1e\n\x12\x61\x63tive_sheet_index\x18\x01 \x02(\rB\x02\x18\x01\x12)\n\rselected_info\x18\x02 \x03(\x0b\x32\x0e.TSP.ReferenceB\x02\x18\x01\x12R\n\x1esheet_uistate_dictionary_entry\x18\x03 \x03(\x0b\x32&.TN.SheetUIStateDictionaryEntryArchiveB\x02\x18\x01\x12\x32\n\x0ftable_selection\x18\x04 \x01(\x0b\x32\x15.TST.SelectionArchiveB\x02\x18\x01\x12\x1f\n\x13\x65\x64iting_sheet_index\x18\x05 \x01(\rB\x02\x18\x01\x12\x15\n\rdocument_mode\x18\x06 \x01(\x05\x12\\\n(edit_mode_sheet_uistate_dictionary_entry\x18\x07 \x03(\x0b\x32&.TN.SheetUIStateDictionaryEntryArchiveB\x02\x18\x01\x12\x1e\n\x12table_editing_mode\x18\x08 \x01(\x05\x42\x02\x18\x01\x12%\n\x19\x66orm_focused_record_index\x18\t \x01(\rB\x02\x18\x01\x12$\n\x18\x66orm_focused_field_index\x18\n \x01(\rB\x02\x18\x01\x12\x15\n\rin_chart_mode\x18\x0b \x01(\x08\x12\x36\n\x0f\x63hart_selection\x18\x0c \x01(\x0b\x32\x19.TN.ChartSelectionArchiveB\x02\x18\x01\x12+\n\x0fsheet_selection\x18\r \x01(\x0b\x32\x0e.TSP.ReferenceB\x02\x18\x01\x12$\n\x16inspector_pane_visible\x18\x0e \x01(\x08:\x04true\x12h\n\x18inspector_pane_view_mode\x18\x0f \x01(\x0e\x32(.TN.UIStateArchive.InspectorPaneViewMode:\x1ckInspectorPaneViewModeFormat\x12%\n\x1dselected_quick_calc_functions\x18\x10 \x03(\r\x12(\n removed_all_quick_calc_functions\x18\x11 \x01(\x08\x12\x1a\n\x12show_canvas_guides\x18\x12 \x01(\x08\x12\x16\n\x0eshows_comments\x18\x13 \x01(\x08\x12)\n\x15\x64\x65sktop_window_origin\x18\x14 \x01(\x0b\x32\n.TSP.Point\x12&\n\x13\x64\x65sktop_window_size\x18\x15 \x01(\x0b\x32\t.TSP.Size\x12&\n\x13\x64\x65sktop_screen_size\x18\x16 \x01(\x0b\x32\t.TSP.Size\x12*\n\x0e\x63hart_ui_state\x18\x17 \x03(\x0b\x32\x12.TSCH.ChartUIState\x12\x31\n\x0eselection_path\x18\x18 \x01(\x0b\x32\x19.TSK.SelectionPathArchive\x12!\n\x19inspector_pane_autohidden\x18\x19 \x01(\x08\x12\x19\n\rshows_sidebar\x18\x1a \x01(\x08\x42\x02\x18\x01\x12\x14\n\x0cshows_rulers\x18\x1b \x01(\x08\x12L\n\x1duuid_sheet_uistate_dictionary\x18\x1c \x03(\x0b\x32%.TN.UUIDSheetUIStateDictionaryArchive\x12\x36\n\x1e\x66reehand_drawing_toolkit_state\x18\x1d \x01(\x0b\x32\x0e.TSP.Reference\x12\x32\n\x1aselection_path_transformer\x18\x1e \x01(\x0b\x32\x0e.TSP.Reference\x12\x18\n\x10\x65\x64iting_disabled\x18\x1f \x01(\x08\"[\n\x15InspectorPaneViewMode\x12 \n\x1ckInspectorPaneViewModeFormat\x10\x00\x12 \n\x1ckInspectorPaneViewModeFilter\x10\x01\"I\n\x15SheetSelectionArchive\x12\x1d\n\x05sheet\x18\x01 \x01(\x0b\x32\x0e.TSP.Reference\x12\x11\n\tpaginated\x18\x02 \x01(\x08\"A\n\x14\x46ormSelectionArchive\x12\x14\n\x0crecord_index\x18\x01 \x02(\r\x12\x13\n\x0b\x66ield_index\x18\x02 \x02(\r\";\n\x1b\x46ormBuilderSelectionArchive\x12\x1c\n\x14viewing_record_index\x18\x01 \x01(\r\"\"\n FormTableChooserSelectionArchive\"<\n\x14UndoRedoStateArchive\x12$\n\x08ui_state\x18\x01 \x02(\x0b\x32\x12.TN.UIStateArchive\"\x89\x03\n\x0f\x44ocumentArchive\x12\x1e\n\x06sheets\x18\x01 \x03(\x0b\x32\x0e.TSP.Reference\x12#\n\x05super\x18\x08 \x02(\x0b\x32\x14.TSA.DocumentArchive\x12.\n\x12\x63\x61lculation_engine\x18\x03 \x01(\x0b\x32\x0e.TSP.ReferenceB\x02\x18\x01\x12\"\n\nstylesheet\x18\x04 \x02(\x0b\x32\x0e.TSP.Reference\x12%\n\rsidebar_order\x18\x05 \x02(\x0b\x32\x0e.TSP.Reference\x12\x1d\n\x05theme\x18\x06 \x02(\x0b\x32\x0e.TSP.Reference\x12#\n\x07uistate\x18\x07 \x01(\x0b\x32\x12.TN.UIStateArchive\x12*\n\x12\x63ustom_format_list\x18\t \x01(\x0b\x32\x0e.TSP.Reference\x12\x16\n\nprinter_id\x18\n \x01(\tB\x02\x18\x01\x12\x10\n\x08paper_id\x18\x0b \x01(\t\x12\x1c\n\tpage_size\x18\x0c \x01(\x0b\x32\t.TSP.Size\";\n\x12PlaceholderArchive\x12%\n\x05super\x18\x01 \x02(\x0b\x32\x16.TSWP.ShapeInfoArchive\"\xb5\x06\n\x0cSheetArchive\x12\x0c\n\x04name\x18\x01 \x02(\t\x12&\n\x0e\x64rawable_infos\x18\x02 \x03(\x0b\x32\x0e.TSP.Reference\x12$\n\x1cin_portrait_page_orientation\x18\x03 \x01(\x08\x12\"\n\x16show_repeating_headers\x18\x04 \x01(\x08\x42\x02\x18\x01\x12\x19\n\x11show_page_numbers\x18\x05 \x01(\x08\x12\x15\n\ris_autofit_on\x18\x06 \x01(\x08\x12\x15\n\rcontent_scale\x18\x07 \x01(\x02\x12!\n\npage_order\x18\x08 \x01(\x0e\x32\r.TN.PageOrder\x12-\n\rprint_margins\x18\n \x01(\x0b\x32\x16.TSD.EdgeInsetsArchive\x12\x1f\n\x17using_start_page_number\x18\x0b \x01(\x08\x12\x19\n\x11start_page_number\x18\x0c \x01(\x05\x12\x19\n\x11page_header_inset\x18\r \x01(\x02\x12\x19\n\x11page_footer_inset\x18\x0e \x01(\x02\x12*\n\x0eheader_storage\x18\x0f \x01(\x0b\x32\x0e.TSP.ReferenceB\x02\x18\x01\x12*\n\x0e\x66ooter_storage\x18\x10 \x01(\x0b\x32\x0e.TSP.ReferenceB\x02\x18\x01\x12/\n\x17userDefinedGuideStorage\x18\x11 \x01(\x0b\x32\x0e.TSP.Reference\x12\x1f\n\x07headers\x18\x12 \x03(\x0b\x32\x0e.TSP.Reference\x12\x1f\n\x07\x66ooters\x18\x13 \x03(\x0b\x32\x0e.TSP.Reference\x12!\n\x19uses_single_header_footer\x18\x14 \x01(\x08\x12Q\n\x10layout_direction\x18\x15 \x01(\x0e\x32\x17.TN.PageLayoutDirection:\x1ePageLayoutDirectionLeftToRight\x12\x1d\n\x05style\x18\x16 \x01(\x0b\x32\x0e.TSP.Reference\x12\x19\n\x11print_backgrounds\x18\x17 \x01(\x08\x12\x1d\n\x15should_print_comments\x18\x18 \x01(\x08\"=\n\x1bSheetStylePropertiesArchive\x12\x1e\n\x04\x66ill\x18\x01 \x01(\x0b\x32\x10.TSD.FillArchive\"\x88\x01\n\x11SheetStyleArchive\x12 \n\x05super\x18\x01 \x02(\x0b\x32\x11.TSS.StyleArchive\x12\x16\n\x0eoverride_count\x18\x02 \x01(\r\x12\x39\n\x10sheet_properties\x18\x03 \x01(\x0b\x32\x1f.TN.SheetStylePropertiesArchive\"^\n\x15\x46ormBasedSheetArchive\x12\x1f\n\x05super\x18\x01 \x02(\x0b\x32\x10.TN.SheetArchive\x12$\n\x08table_id\x18\x02 \x01(\x0b\x32\x12.TSP.CFUUIDArchive\"T\n\x0cThemeArchive\x12 \n\x05super\x18\x01 \x02(\x0b\x32\x11.TSS.ThemeArchive\x12\"\n\nprototypes\x18\x02 \x03(\x0b\x32\x0e.TSP.Reference\"j\n\x1ePasteboardNativeStorageArchive\x12\x1d\n\x05sheet\x18\x01 \x01(\x0b\x32\x0e.TSP.Reference\x12)\n\x08ui_state\x18\x02 \x01(\x0b\x32\x17.TN.SheetUIStateArchive\"\xc7\x03\n\x1b\x43hartMediatorFormulaStorage\x12+\n\rdata_formulae\x18\x01 \x03(\x0b\x32\x14.TSCE.FormulaArchive\x12\x30\n\x12row_label_formulae\x18\x03 \x03(\x0b\x32\x14.TSCE.FormulaArchive\x12\x30\n\x12\x63ol_label_formulae\x18\x04 \x03(\x0b\x32\x14.TSCE.FormulaArchive\x12\x11\n\tdirection\x18\x05 \x01(\x05\x12\x37\n\x19\x65rror_custom_pos_formulae\x18\x06 \x03(\x0b\x32\x14.TSCE.FormulaArchive\x12\x37\n\x19\x65rror_custom_neg_formulae\x18\x07 \x03(\x0b\x32\x14.TSCE.FormulaArchive\x12@\n\"error_custom_pos_scatterX_formulae\x18\x08 \x03(\x0b\x32\x14.TSCE.FormulaArchive\x12@\n\"error_custom_neg_scatterX_formulae\x18\t \x03(\x0b\x32\x14.TSCE.FormulaArchive\x12\x0e\n\x06scheme\x18\n \x01(\x05\"\xcf\x01\n\x14\x43hartMediatorArchive\x12)\n\x05super\x18\x01 \x02(\x0b\x32\x1a.TSCH.ChartMediatorArchive\x12\x11\n\tentity_id\x18\x02 \x02(\t\x12\x31\n\x08\x66ormulas\x18\x03 \x01(\x0b\x32\x1f.TN.ChartMediatorFormulaStorage\x12\x1a\n\x12\x63olumns_are_series\x18\x04 \x01(\x08\x12*\n\x1eis_registered_with_calc_engine\x18\x05 \x01(\x08\x42\x02\x18\x01\"\xcf\x01\n\x15\x43hartSelectionArchive\x12.\n\treference\x18\x01 \x01(\x0b\x32\x1b.TSCE.RangeReferenceArchive\x12\x39\n\x10\x64\x65precated_super\x18\x02 \x01(\x0b\x32\x1b.TSCH.ChartSelectionArchiveB\x02\x18\x01\x12\x1d\n\x05\x63hart\x18\x03 \x01(\x0b\x32\x0e.TSP.Reference\x12,\n\x05super\x18\x04 \x01(\x0b\x32\x1d.TSD.DrawableSelectionArchive*?\n\tPageOrder\x12\x18\n\x14PageOrderTopToBottom\x10\x00\x12\x18\n\x14PageOrderLeftToRight\x10\x01*]\n\x13PageLayoutDirection\x12\"\n\x1ePageLayoutDirectionLeftToRight\x10\x00\x12\"\n\x1ePageLayoutDirectionRightToLeft\x10\x01'
  ,
  dependencies=[TSPMessages__pb2.DESCRIPTOR,TSKArchives__pb2.DESCRIPTOR,TSCHArchives__pb2.DESCRIPTOR,TSCEArchives__pb2.DESCRIPTOR,TSSArchives__pb2.DESCRIPTOR,TSDArchives__pb2.DESCRIPTOR,TSWPArchives__pb2.DESCRIPTOR,TSAArchives__pb2.DESCRIPTOR,TSTArchives__pb2.DESCRIPTOR,])

_PAGEORDER = _descriptor.EnumDescriptor(
  name='PageOrder',
  full_name='TN.PageOrder',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PageOrderTopToBottom', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PageOrderLeftToRight', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=5344,
  serialized_end=5407,
)
_sym_db.RegisterEnumDescriptor(_PAGEORDER)

PageOrder = enum_type_wrapper.EnumTypeWrapper(_PAGEORDER)
_PAGELAYOUTDIRECTION = _descriptor.EnumDescriptor(
  name='PageLayoutDirection',
  full_name='TN.PageLayoutDirection',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='PageLayoutDirectionLeftToRight', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PageLayoutDirectionRightToLeft', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=5409,
  serialized_end=5502,
)
_sym_db.RegisterEnumDescriptor(_PAGELAYOUTDIRECTION)

PageLayoutDirection = enum_type_wrapper.EnumTypeWrapper(_PAGELAYOUTDIRECTION)
PageOrderTopToBottom = 0
PageOrderLeftToRight = 1
PageLayoutDirectionLeftToRight = 0
PageLayoutDirectionRightToLeft = 1


_UISTATEARCHIVE_INSPECTORPANEVIEWMODE = _descriptor.EnumDescriptor(
  name='InspectorPaneViewMode',
  full_name='TN.UIStateArchive.InspectorPaneViewMode',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='kInspectorPaneViewModeFormat', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='kInspectorPaneViewModeFilter', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=2299,
  serialized_end=2390,
)
_sym_db.RegisterEnumDescriptor(_UISTATEARCHIVE_INSPECTORPANEVIEWMODE)


_SHEETUISTATEARCHIVE = _descriptor.Descriptor(
  name='SheetUIStateArchive',
  full_name='TN.SheetUIStateArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='view_scale', full_name='TN.SheetUIStateArchive.view_scale', index=0,
      number=1, type=2, cpp_type=6, label=2,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scroll_position', full_name='TN.SheetUIStateArchive.scroll_position', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='previous_view_scale', full_name='TN.SheetUIStateArchive.previous_view_scale', index=2,
      number=3, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scroll_position_is_unscaled', full_name='TN.SheetUIStateArchive.scroll_position_is_unscaled', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='previous_scroll_position', full_name='TN.SheetUIStateArchive.previous_scroll_position', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scroll_position_valid', full_name='TN.SheetUIStateArchive.scroll_position_valid', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='previous_scroll_position_valid', full_name='TN.SheetUIStateArchive.previous_scroll_position_valid', index=6,
      number=7, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='visible_size', full_name='TN.SheetUIStateArchive.visible_size', index=7,
      number=8, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='previous_visible_size', full_name='TN.SheetUIStateArchive.previous_visible_size', index=8,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='device_idiom', full_name='TN.SheetUIStateArchive.device_idiom', index=9,
      number=10, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selection_path', full_name='TN.SheetUIStateArchive.selection_path', index=10,
      number=11, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selection_path_transformer', full_name='TN.SheetUIStateArchive.selection_path_transformer', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=199,
  serialized_end=660,
)


_SHEETUISTATEDICTIONARYENTRYARCHIVE = _descriptor.Descriptor(
  name='SheetUIStateDictionaryEntryArchive',
  full_name='TN.SheetUIStateDictionaryEntryArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sheet', full_name='TN.SheetUIStateDictionaryEntryArchive.sheet', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sheet_uistate', full_name='TN.SheetUIStateDictionaryEntryArchive.sheet_uistate', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=662,
  serialized_end=777,
)


_UUIDSHEETUISTATEDICTIONARYARCHIVE = _descriptor.Descriptor(
  name='UUIDSheetUIStateDictionaryArchive',
  full_name='TN.UUIDSheetUIStateDictionaryArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sheet_uuid', full_name='TN.UUIDSheetUIStateDictionaryArchive.sheet_uuid', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sheet_uistate', full_name='TN.UUIDSheetUIStateDictionaryArchive.sheet_uistate', index=1,
      number=2, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=779,
  serialized_end=893,
)


_UISTATEARCHIVE = _descriptor.Descriptor(
  name='UIStateArchive',
  full_name='TN.UIStateArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='active_sheet_index', full_name='TN.UIStateArchive.active_sheet_index', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selected_info', full_name='TN.UIStateArchive.selected_info', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sheet_uistate_dictionary_entry', full_name='TN.UIStateArchive.sheet_uistate_dictionary_entry', index=2,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_selection', full_name='TN.UIStateArchive.table_selection', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='editing_sheet_index', full_name='TN.UIStateArchive.editing_sheet_index', index=4,
      number=5, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='document_mode', full_name='TN.UIStateArchive.document_mode', index=5,
      number=6, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='edit_mode_sheet_uistate_dictionary_entry', full_name='TN.UIStateArchive.edit_mode_sheet_uistate_dictionary_entry', index=6,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_editing_mode', full_name='TN.UIStateArchive.table_editing_mode', index=7,
      number=8, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='form_focused_record_index', full_name='TN.UIStateArchive.form_focused_record_index', index=8,
      number=9, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='form_focused_field_index', full_name='TN.UIStateArchive.form_focused_field_index', index=9,
      number=10, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='in_chart_mode', full_name='TN.UIStateArchive.in_chart_mode', index=10,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='chart_selection', full_name='TN.UIStateArchive.chart_selection', index=11,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sheet_selection', full_name='TN.UIStateArchive.sheet_selection', index=12,
      number=13, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inspector_pane_visible', full_name='TN.UIStateArchive.inspector_pane_visible', index=13,
      number=14, type=8, cpp_type=7, label=1,
      has_default_value=True, default_value=True,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inspector_pane_view_mode', full_name='TN.UIStateArchive.inspector_pane_view_mode', index=14,
      number=15, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selected_quick_calc_functions', full_name='TN.UIStateArchive.selected_quick_calc_functions', index=15,
      number=16, type=13, cpp_type=3, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='removed_all_quick_calc_functions', full_name='TN.UIStateArchive.removed_all_quick_calc_functions', index=16,
      number=17, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='show_canvas_guides', full_name='TN.UIStateArchive.show_canvas_guides', index=17,
      number=18, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shows_comments', full_name='TN.UIStateArchive.shows_comments', index=18,
      number=19, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='desktop_window_origin', full_name='TN.UIStateArchive.desktop_window_origin', index=19,
      number=20, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='desktop_window_size', full_name='TN.UIStateArchive.desktop_window_size', index=20,
      number=21, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='desktop_screen_size', full_name='TN.UIStateArchive.desktop_screen_size', index=21,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='chart_ui_state', full_name='TN.UIStateArchive.chart_ui_state', index=22,
      number=23, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selection_path', full_name='TN.UIStateArchive.selection_path', index=23,
      number=24, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='inspector_pane_autohidden', full_name='TN.UIStateArchive.inspector_pane_autohidden', index=24,
      number=25, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shows_sidebar', full_name='TN.UIStateArchive.shows_sidebar', index=25,
      number=26, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='shows_rulers', full_name='TN.UIStateArchive.shows_rulers', index=26,
      number=27, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uuid_sheet_uistate_dictionary', full_name='TN.UIStateArchive.uuid_sheet_uistate_dictionary', index=27,
      number=28, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='freehand_drawing_toolkit_state', full_name='TN.UIStateArchive.freehand_drawing_toolkit_state', index=28,
      number=29, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='selection_path_transformer', full_name='TN.UIStateArchive.selection_path_transformer', index=29,
      number=30, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='editing_disabled', full_name='TN.UIStateArchive.editing_disabled', index=30,
      number=31, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
    _UISTATEARCHIVE_INSPECTORPANEVIEWMODE,
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=896,
  serialized_end=2390,
)


_SHEETSELECTIONARCHIVE = _descriptor.Descriptor(
  name='SheetSelectionArchive',
  full_name='TN.SheetSelectionArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sheet', full_name='TN.SheetSelectionArchive.sheet', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='paginated', full_name='TN.SheetSelectionArchive.paginated', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2392,
  serialized_end=2465,
)


_FORMSELECTIONARCHIVE = _descriptor.Descriptor(
  name='FormSelectionArchive',
  full_name='TN.FormSelectionArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='record_index', full_name='TN.FormSelectionArchive.record_index', index=0,
      number=1, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='field_index', full_name='TN.FormSelectionArchive.field_index', index=1,
      number=2, type=13, cpp_type=3, label=2,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2467,
  serialized_end=2532,
)


_FORMBUILDERSELECTIONARCHIVE = _descriptor.Descriptor(
  name='FormBuilderSelectionArchive',
  full_name='TN.FormBuilderSelectionArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='viewing_record_index', full_name='TN.FormBuilderSelectionArchive.viewing_record_index', index=0,
      number=1, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2534,
  serialized_end=2593,
)


_FORMTABLECHOOSERSELECTIONARCHIVE = _descriptor.Descriptor(
  name='FormTableChooserSelectionArchive',
  full_name='TN.FormTableChooserSelectionArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2595,
  serialized_end=2629,
)


_UNDOREDOSTATEARCHIVE = _descriptor.Descriptor(
  name='UndoRedoStateArchive',
  full_name='TN.UndoRedoStateArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='ui_state', full_name='TN.UndoRedoStateArchive.ui_state', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2631,
  serialized_end=2691,
)


_DOCUMENTARCHIVE = _descriptor.Descriptor(
  name='DocumentArchive',
  full_name='TN.DocumentArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sheets', full_name='TN.DocumentArchive.sheets', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='super', full_name='TN.DocumentArchive.super', index=1,
      number=8, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='calculation_engine', full_name='TN.DocumentArchive.calculation_engine', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='stylesheet', full_name='TN.DocumentArchive.stylesheet', index=3,
      number=4, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sidebar_order', full_name='TN.DocumentArchive.sidebar_order', index=4,
      number=5, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='theme', full_name='TN.DocumentArchive.theme', index=5,
      number=6, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uistate', full_name='TN.DocumentArchive.uistate', index=6,
      number=7, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='custom_format_list', full_name='TN.DocumentArchive.custom_format_list', index=7,
      number=9, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='printer_id', full_name='TN.DocumentArchive.printer_id', index=8,
      number=10, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='paper_id', full_name='TN.DocumentArchive.paper_id', index=9,
      number=11, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='TN.DocumentArchive.page_size', index=10,
      number=12, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=2694,
  serialized_end=3087,
)


_PLACEHOLDERARCHIVE = _descriptor.Descriptor(
  name='PlaceholderArchive',
  full_name='TN.PlaceholderArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TN.PlaceholderArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3089,
  serialized_end=3148,
)


_SHEETARCHIVE = _descriptor.Descriptor(
  name='SheetArchive',
  full_name='TN.SheetArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='name', full_name='TN.SheetArchive.name', index=0,
      number=1, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='drawable_infos', full_name='TN.SheetArchive.drawable_infos', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='in_portrait_page_orientation', full_name='TN.SheetArchive.in_portrait_page_orientation', index=2,
      number=3, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='show_repeating_headers', full_name='TN.SheetArchive.show_repeating_headers', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='show_page_numbers', full_name='TN.SheetArchive.show_page_numbers', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_autofit_on', full_name='TN.SheetArchive.is_autofit_on', index=5,
      number=6, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='content_scale', full_name='TN.SheetArchive.content_scale', index=6,
      number=7, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_order', full_name='TN.SheetArchive.page_order', index=7,
      number=8, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='print_margins', full_name='TN.SheetArchive.print_margins', index=8,
      number=10, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='using_start_page_number', full_name='TN.SheetArchive.using_start_page_number', index=9,
      number=11, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='start_page_number', full_name='TN.SheetArchive.start_page_number', index=10,
      number=12, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_header_inset', full_name='TN.SheetArchive.page_header_inset', index=11,
      number=13, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_footer_inset', full_name='TN.SheetArchive.page_footer_inset', index=12,
      number=14, type=2, cpp_type=6, label=1,
      has_default_value=False, default_value=float(0),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='header_storage', full_name='TN.SheetArchive.header_storage', index=13,
      number=15, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='footer_storage', full_name='TN.SheetArchive.footer_storage', index=14,
      number=16, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='userDefinedGuideStorage', full_name='TN.SheetArchive.userDefinedGuideStorage', index=15,
      number=17, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='headers', full_name='TN.SheetArchive.headers', index=16,
      number=18, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='footers', full_name='TN.SheetArchive.footers', index=17,
      number=19, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='uses_single_header_footer', full_name='TN.SheetArchive.uses_single_header_footer', index=18,
      number=20, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='layout_direction', full_name='TN.SheetArchive.layout_direction', index=19,
      number=21, type=14, cpp_type=8, label=1,
      has_default_value=True, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='style', full_name='TN.SheetArchive.style', index=20,
      number=22, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='print_backgrounds', full_name='TN.SheetArchive.print_backgrounds', index=21,
      number=23, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='should_print_comments', full_name='TN.SheetArchive.should_print_comments', index=22,
      number=24, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3151,
  serialized_end=3972,
)


_SHEETSTYLEPROPERTIESARCHIVE = _descriptor.Descriptor(
  name='SheetStylePropertiesArchive',
  full_name='TN.SheetStylePropertiesArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='fill', full_name='TN.SheetStylePropertiesArchive.fill', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=3974,
  serialized_end=4035,
)


_SHEETSTYLEARCHIVE = _descriptor.Descriptor(
  name='SheetStyleArchive',
  full_name='TN.SheetStyleArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TN.SheetStyleArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='override_count', full_name='TN.SheetStyleArchive.override_count', index=1,
      number=2, type=13, cpp_type=3, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='sheet_properties', full_name='TN.SheetStyleArchive.sheet_properties', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4038,
  serialized_end=4174,
)


_FORMBASEDSHEETARCHIVE = _descriptor.Descriptor(
  name='FormBasedSheetArchive',
  full_name='TN.FormBasedSheetArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TN.FormBasedSheetArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='table_id', full_name='TN.FormBasedSheetArchive.table_id', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4176,
  serialized_end=4270,
)


_THEMEARCHIVE = _descriptor.Descriptor(
  name='ThemeArchive',
  full_name='TN.ThemeArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TN.ThemeArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='prototypes', full_name='TN.ThemeArchive.prototypes', index=1,
      number=2, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4272,
  serialized_end=4356,
)


_PASTEBOARDNATIVESTORAGEARCHIVE = _descriptor.Descriptor(
  name='PasteboardNativeStorageArchive',
  full_name='TN.PasteboardNativeStorageArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='sheet', full_name='TN.PasteboardNativeStorageArchive.sheet', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='ui_state', full_name='TN.PasteboardNativeStorageArchive.ui_state', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4358,
  serialized_end=4464,
)


_CHARTMEDIATORFORMULASTORAGE = _descriptor.Descriptor(
  name='ChartMediatorFormulaStorage',
  full_name='TN.ChartMediatorFormulaStorage',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='data_formulae', full_name='TN.ChartMediatorFormulaStorage.data_formulae', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='row_label_formulae', full_name='TN.ChartMediatorFormulaStorage.row_label_formulae', index=1,
      number=3, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='col_label_formulae', full_name='TN.ChartMediatorFormulaStorage.col_label_formulae', index=2,
      number=4, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='direction', full_name='TN.ChartMediatorFormulaStorage.direction', index=3,
      number=5, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error_custom_pos_formulae', full_name='TN.ChartMediatorFormulaStorage.error_custom_pos_formulae', index=4,
      number=6, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error_custom_neg_formulae', full_name='TN.ChartMediatorFormulaStorage.error_custom_neg_formulae', index=5,
      number=7, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error_custom_pos_scatterX_formulae', full_name='TN.ChartMediatorFormulaStorage.error_custom_pos_scatterX_formulae', index=6,
      number=8, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error_custom_neg_scatterX_formulae', full_name='TN.ChartMediatorFormulaStorage.error_custom_neg_scatterX_formulae', index=7,
      number=9, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='scheme', full_name='TN.ChartMediatorFormulaStorage.scheme', index=8,
      number=10, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4467,
  serialized_end=4922,
)


_CHARTMEDIATORARCHIVE = _descriptor.Descriptor(
  name='ChartMediatorArchive',
  full_name='TN.ChartMediatorArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='super', full_name='TN.ChartMediatorArchive.super', index=0,
      number=1, type=11, cpp_type=10, label=2,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='entity_id', full_name='TN.ChartMediatorArchive.entity_id', index=1,
      number=2, type=9, cpp_type=9, label=2,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='formulas', full_name='TN.ChartMediatorArchive.formulas', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='columns_are_series', full_name='TN.ChartMediatorArchive.columns_are_series', index=3,
      number=4, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='is_registered_with_calc_engine', full_name='TN.ChartMediatorArchive.is_registered_with_calc_engine', index=4,
      number=5, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=4925,
  serialized_end=5132,
)


_CHARTSELECTIONARCHIVE = _descriptor.Descriptor(
  name='ChartSelectionArchive',
  full_name='TN.ChartSelectionArchive',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='reference', full_name='TN.ChartSelectionArchive.reference', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='deprecated_super', full_name='TN.ChartSelectionArchive.deprecated_super', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=b'\030\001', file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='chart', full_name='TN.ChartSelectionArchive.chart', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='super', full_name='TN.ChartSelectionArchive.super', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto2',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=5135,
  serialized_end=5342,
)

_SHEETUISTATEARCHIVE.fields_by_name['scroll_position'].message_type = TSPMessages__pb2._POINT
_SHEETUISTATEARCHIVE.fields_by_name['previous_scroll_position'].message_type = TSPMessages__pb2._POINT
_SHEETUISTATEARCHIVE.fields_by_name['visible_size'].message_type = TSPMessages__pb2._SIZE
_SHEETUISTATEARCHIVE.fields_by_name['previous_visible_size'].message_type = TSPMessages__pb2._SIZE
_SHEETUISTATEARCHIVE.fields_by_name['selection_path'].message_type = TSKArchives__pb2._SELECTIONPATHARCHIVE
_SHEETUISTATEARCHIVE.fields_by_name['selection_path_transformer'].message_type = TSPMessages__pb2._REFERENCE
_SHEETUISTATEDICTIONARYENTRYARCHIVE.fields_by_name['sheet'].message_type = TSPMessages__pb2._REFERENCE
_SHEETUISTATEDICTIONARYENTRYARCHIVE.fields_by_name['sheet_uistate'].message_type = _SHEETUISTATEARCHIVE
_UUIDSHEETUISTATEDICTIONARYARCHIVE.fields_by_name['sheet_uuid'].message_type = TSPMessages__pb2._UUID
_UUIDSHEETUISTATEDICTIONARYARCHIVE.fields_by_name['sheet_uistate'].message_type = _SHEETUISTATEARCHIVE
_UISTATEARCHIVE.fields_by_name['selected_info'].message_type = TSPMessages__pb2._REFERENCE
_UISTATEARCHIVE.fields_by_name['sheet_uistate_dictionary_entry'].message_type = _SHEETUISTATEDICTIONARYENTRYARCHIVE
_UISTATEARCHIVE.fields_by_name['table_selection'].message_type = TSTArchives__pb2._SELECTIONARCHIVE
_UISTATEARCHIVE.fields_by_name['edit_mode_sheet_uistate_dictionary_entry'].message_type = _SHEETUISTATEDICTIONARYENTRYARCHIVE
_UISTATEARCHIVE.fields_by_name['chart_selection'].message_type = _CHARTSELECTIONARCHIVE
_UISTATEARCHIVE.fields_by_name['sheet_selection'].message_type = TSPMessages__pb2._REFERENCE
_UISTATEARCHIVE.fields_by_name['inspector_pane_view_mode'].enum_type = _UISTATEARCHIVE_INSPECTORPANEVIEWMODE
_UISTATEARCHIVE.fields_by_name['desktop_window_origin'].message_type = TSPMessages__pb2._POINT
_UISTATEARCHIVE.fields_by_name['desktop_window_size'].message_type = TSPMessages__pb2._SIZE
_UISTATEARCHIVE.fields_by_name['desktop_screen_size'].message_type = TSPMessages__pb2._SIZE
_UISTATEARCHIVE.fields_by_name['chart_ui_state'].message_type = TSCHArchives__pb2._CHARTUISTATE
_UISTATEARCHIVE.fields_by_name['selection_path'].message_type = TSKArchives__pb2._SELECTIONPATHARCHIVE
_UISTATEARCHIVE.fields_by_name['uuid_sheet_uistate_dictionary'].message_type = _UUIDSHEETUISTATEDICTIONARYARCHIVE
_UISTATEARCHIVE.fields_by_name['freehand_drawing_toolkit_state'].message_type = TSPMessages__pb2._REFERENCE
_UISTATEARCHIVE.fields_by_name['selection_path_transformer'].message_type = TSPMessages__pb2._REFERENCE
_UISTATEARCHIVE_INSPECTORPANEVIEWMODE.containing_type = _UISTATEARCHIVE
_SHEETSELECTIONARCHIVE.fields_by_name['sheet'].message_type = TSPMessages__pb2._REFERENCE
_UNDOREDOSTATEARCHIVE.fields_by_name['ui_state'].message_type = _UISTATEARCHIVE
_DOCUMENTARCHIVE.fields_by_name['sheets'].message_type = TSPMessages__pb2._REFERENCE
_DOCUMENTARCHIVE.fields_by_name['super'].message_type = TSAArchives__pb2._DOCUMENTARCHIVE
_DOCUMENTARCHIVE.fields_by_name['calculation_engine'].message_type = TSPMessages__pb2._REFERENCE
_DOCUMENTARCHIVE.fields_by_name['stylesheet'].message_type = TSPMessages__pb2._REFERENCE
_DOCUMENTARCHIVE.fields_by_name['sidebar_order'].message_type = TSPMessages__pb2._REFERENCE
_DOCUMENTARCHIVE.fields_by_name['theme'].message_type = TSPMessages__pb2._REFERENCE
_DOCUMENTARCHIVE.fields_by_name['uistate'].message_type = _UISTATEARCHIVE
_DOCUMENTARCHIVE.fields_by_name['custom_format_list'].message_type = TSPMessages__pb2._REFERENCE
_DOCUMENTARCHIVE.fields_by_name['page_size'].message_type = TSPMessages__pb2._SIZE
_PLACEHOLDERARCHIVE.fields_by_name['super'].message_type = TSWPArchives__pb2._SHAPEINFOARCHIVE
_SHEETARCHIVE.fields_by_name['drawable_infos'].message_type = TSPMessages__pb2._REFERENCE
_SHEETARCHIVE.fields_by_name['page_order'].enum_type = _PAGEORDER
_SHEETARCHIVE.fields_by_name['print_margins'].message_type = TSDArchives__pb2._EDGEINSETSARCHIVE
_SHEETARCHIVE.fields_by_name['header_storage'].message_type = TSPMessages__pb2._REFERENCE
_SHEETARCHIVE.fields_by_name['footer_storage'].message_type = TSPMessages__pb2._REFERENCE
_SHEETARCHIVE.fields_by_name['userDefinedGuideStorage'].message_type = TSPMessages__pb2._REFERENCE
_SHEETARCHIVE.fields_by_name['headers'].message_type = TSPMessages__pb2._REFERENCE
_SHEETARCHIVE.fields_by_name['footers'].message_type = TSPMessages__pb2._REFERENCE
_SHEETARCHIVE.fields_by_name['layout_direction'].enum_type = _PAGELAYOUTDIRECTION
_SHEETARCHIVE.fields_by_name['style'].message_type = TSPMessages__pb2._REFERENCE
_SHEETSTYLEPROPERTIESARCHIVE.fields_by_name['fill'].message_type = TSDArchives__pb2._FILLARCHIVE
_SHEETSTYLEARCHIVE.fields_by_name['super'].message_type = TSSArchives__pb2._STYLEARCHIVE
_SHEETSTYLEARCHIVE.fields_by_name['sheet_properties'].message_type = _SHEETSTYLEPROPERTIESARCHIVE
_FORMBASEDSHEETARCHIVE.fields_by_name['super'].message_type = _SHEETARCHIVE
_FORMBASEDSHEETARCHIVE.fields_by_name['table_id'].message_type = TSPMessages__pb2._CFUUIDARCHIVE
_THEMEARCHIVE.fields_by_name['super'].message_type = TSSArchives__pb2._THEMEARCHIVE
_THEMEARCHIVE.fields_by_name['prototypes'].message_type = TSPMessages__pb2._REFERENCE
_PASTEBOARDNATIVESTORAGEARCHIVE.fields_by_name['sheet'].message_type = TSPMessages__pb2._REFERENCE
_PASTEBOARDNATIVESTORAGEARCHIVE.fields_by_name['ui_state'].message_type = _SHEETUISTATEARCHIVE
_CHARTMEDIATORFORMULASTORAGE.fields_by_name['data_formulae'].message_type = TSCEArchives__pb2._FORMULAARCHIVE
_CHARTMEDIATORFORMULASTORAGE.fields_by_name['row_label_formulae'].message_type = TSCEArchives__pb2._FORMULAARCHIVE
_CHARTMEDIATORFORMULASTORAGE.fields_by_name['col_label_formulae'].message_type = TSCEArchives__pb2._FORMULAARCHIVE
_CHARTMEDIATORFORMULASTORAGE.fields_by_name['error_custom_pos_formulae'].message_type = TSCEArchives__pb2._FORMULAARCHIVE
_CHARTMEDIATORFORMULASTORAGE.fields_by_name['error_custom_neg_formulae'].message_type = TSCEArchives__pb2._FORMULAARCHIVE
_CHARTMEDIATORFORMULASTORAGE.fields_by_name['error_custom_pos_scatterX_formulae'].message_type = TSCEArchives__pb2._FORMULAARCHIVE
_CHARTMEDIATORFORMULASTORAGE.fields_by_name['error_custom_neg_scatterX_formulae'].message_type = TSCEArchives__pb2._FORMULAARCHIVE
_CHARTMEDIATORARCHIVE.fields_by_name['super'].message_type = TSCHArchives__pb2._CHARTMEDIATORARCHIVE
_CHARTMEDIATORARCHIVE.fields_by_name['formulas'].message_type = _CHARTMEDIATORFORMULASTORAGE
_CHARTSELECTIONARCHIVE.fields_by_name['reference'].message_type = TSCEArchives__pb2._RANGEREFERENCEARCHIVE
_CHARTSELECTIONARCHIVE.fields_by_name['deprecated_super'].message_type = TSCHArchives__pb2._CHARTSELECTIONARCHIVE
_CHARTSELECTIONARCHIVE.fields_by_name['chart'].message_type = TSPMessages__pb2._REFERENCE
_CHARTSELECTIONARCHIVE.fields_by_name['super'].message_type = TSDArchives__pb2._DRAWABLESELECTIONARCHIVE
DESCRIPTOR.message_types_by_name['SheetUIStateArchive'] = _SHEETUISTATEARCHIVE
DESCRIPTOR.message_types_by_name['SheetUIStateDictionaryEntryArchive'] = _SHEETUISTATEDICTIONARYENTRYARCHIVE
DESCRIPTOR.message_types_by_name['UUIDSheetUIStateDictionaryArchive'] = _UUIDSHEETUISTATEDICTIONARYARCHIVE
DESCRIPTOR.message_types_by_name['UIStateArchive'] = _UISTATEARCHIVE
DESCRIPTOR.message_types_by_name['SheetSelectionArchive'] = _SHEETSELECTIONARCHIVE
DESCRIPTOR.message_types_by_name['FormSelectionArchive'] = _FORMSELECTIONARCHIVE
DESCRIPTOR.message_types_by_name['FormBuilderSelectionArchive'] = _FORMBUILDERSELECTIONARCHIVE
DESCRIPTOR.message_types_by_name['FormTableChooserSelectionArchive'] = _FORMTABLECHOOSERSELECTIONARCHIVE
DESCRIPTOR.message_types_by_name['UndoRedoStateArchive'] = _UNDOREDOSTATEARCHIVE
DESCRIPTOR.message_types_by_name['DocumentArchive'] = _DOCUMENTARCHIVE
DESCRIPTOR.message_types_by_name['PlaceholderArchive'] = _PLACEHOLDERARCHIVE
DESCRIPTOR.message_types_by_name['SheetArchive'] = _SHEETARCHIVE
DESCRIPTOR.message_types_by_name['SheetStylePropertiesArchive'] = _SHEETSTYLEPROPERTIESARCHIVE
DESCRIPTOR.message_types_by_name['SheetStyleArchive'] = _SHEETSTYLEARCHIVE
DESCRIPTOR.message_types_by_name['FormBasedSheetArchive'] = _FORMBASEDSHEETARCHIVE
DESCRIPTOR.message_types_by_name['ThemeArchive'] = _THEMEARCHIVE
DESCRIPTOR.message_types_by_name['PasteboardNativeStorageArchive'] = _PASTEBOARDNATIVESTORAGEARCHIVE
DESCRIPTOR.message_types_by_name['ChartMediatorFormulaStorage'] = _CHARTMEDIATORFORMULASTORAGE
DESCRIPTOR.message_types_by_name['ChartMediatorArchive'] = _CHARTMEDIATORARCHIVE
DESCRIPTOR.message_types_by_name['ChartSelectionArchive'] = _CHARTSELECTIONARCHIVE
DESCRIPTOR.enum_types_by_name['PageOrder'] = _PAGEORDER
DESCRIPTOR.enum_types_by_name['PageLayoutDirection'] = _PAGELAYOUTDIRECTION
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

SheetUIStateArchive = _reflection.GeneratedProtocolMessageType('SheetUIStateArchive', (_message.Message,), {
  'DESCRIPTOR' : _SHEETUISTATEARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.SheetUIStateArchive)
  })
_sym_db.RegisterMessage(SheetUIStateArchive)

SheetUIStateDictionaryEntryArchive = _reflection.GeneratedProtocolMessageType('SheetUIStateDictionaryEntryArchive', (_message.Message,), {
  'DESCRIPTOR' : _SHEETUISTATEDICTIONARYENTRYARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.SheetUIStateDictionaryEntryArchive)
  })
_sym_db.RegisterMessage(SheetUIStateDictionaryEntryArchive)

UUIDSheetUIStateDictionaryArchive = _reflection.GeneratedProtocolMessageType('UUIDSheetUIStateDictionaryArchive', (_message.Message,), {
  'DESCRIPTOR' : _UUIDSHEETUISTATEDICTIONARYARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.UUIDSheetUIStateDictionaryArchive)
  })
_sym_db.RegisterMessage(UUIDSheetUIStateDictionaryArchive)

UIStateArchive = _reflection.GeneratedProtocolMessageType('UIStateArchive', (_message.Message,), {
  'DESCRIPTOR' : _UISTATEARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.UIStateArchive)
  })
_sym_db.RegisterMessage(UIStateArchive)

SheetSelectionArchive = _reflection.GeneratedProtocolMessageType('SheetSelectionArchive', (_message.Message,), {
  'DESCRIPTOR' : _SHEETSELECTIONARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.SheetSelectionArchive)
  })
_sym_db.RegisterMessage(SheetSelectionArchive)

FormSelectionArchive = _reflection.GeneratedProtocolMessageType('FormSelectionArchive', (_message.Message,), {
  'DESCRIPTOR' : _FORMSELECTIONARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.FormSelectionArchive)
  })
_sym_db.RegisterMessage(FormSelectionArchive)

FormBuilderSelectionArchive = _reflection.GeneratedProtocolMessageType('FormBuilderSelectionArchive', (_message.Message,), {
  'DESCRIPTOR' : _FORMBUILDERSELECTIONARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.FormBuilderSelectionArchive)
  })
_sym_db.RegisterMessage(FormBuilderSelectionArchive)

FormTableChooserSelectionArchive = _reflection.GeneratedProtocolMessageType('FormTableChooserSelectionArchive', (_message.Message,), {
  'DESCRIPTOR' : _FORMTABLECHOOSERSELECTIONARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.FormTableChooserSelectionArchive)
  })
_sym_db.RegisterMessage(FormTableChooserSelectionArchive)

UndoRedoStateArchive = _reflection.GeneratedProtocolMessageType('UndoRedoStateArchive', (_message.Message,), {
  'DESCRIPTOR' : _UNDOREDOSTATEARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.UndoRedoStateArchive)
  })
_sym_db.RegisterMessage(UndoRedoStateArchive)

DocumentArchive = _reflection.GeneratedProtocolMessageType('DocumentArchive', (_message.Message,), {
  'DESCRIPTOR' : _DOCUMENTARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.DocumentArchive)
  })
_sym_db.RegisterMessage(DocumentArchive)

PlaceholderArchive = _reflection.GeneratedProtocolMessageType('PlaceholderArchive', (_message.Message,), {
  'DESCRIPTOR' : _PLACEHOLDERARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.PlaceholderArchive)
  })
_sym_db.RegisterMessage(PlaceholderArchive)

SheetArchive = _reflection.GeneratedProtocolMessageType('SheetArchive', (_message.Message,), {
  'DESCRIPTOR' : _SHEETARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.SheetArchive)
  })
_sym_db.RegisterMessage(SheetArchive)

SheetStylePropertiesArchive = _reflection.GeneratedProtocolMessageType('SheetStylePropertiesArchive', (_message.Message,), {
  'DESCRIPTOR' : _SHEETSTYLEPROPERTIESARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.SheetStylePropertiesArchive)
  })
_sym_db.RegisterMessage(SheetStylePropertiesArchive)

SheetStyleArchive = _reflection.GeneratedProtocolMessageType('SheetStyleArchive', (_message.Message,), {
  'DESCRIPTOR' : _SHEETSTYLEARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.SheetStyleArchive)
  })
_sym_db.RegisterMessage(SheetStyleArchive)

FormBasedSheetArchive = _reflection.GeneratedProtocolMessageType('FormBasedSheetArchive', (_message.Message,), {
  'DESCRIPTOR' : _FORMBASEDSHEETARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.FormBasedSheetArchive)
  })
_sym_db.RegisterMessage(FormBasedSheetArchive)

ThemeArchive = _reflection.GeneratedProtocolMessageType('ThemeArchive', (_message.Message,), {
  'DESCRIPTOR' : _THEMEARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.ThemeArchive)
  })
_sym_db.RegisterMessage(ThemeArchive)

PasteboardNativeStorageArchive = _reflection.GeneratedProtocolMessageType('PasteboardNativeStorageArchive', (_message.Message,), {
  'DESCRIPTOR' : _PASTEBOARDNATIVESTORAGEARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.PasteboardNativeStorageArchive)
  })
_sym_db.RegisterMessage(PasteboardNativeStorageArchive)

ChartMediatorFormulaStorage = _reflection.GeneratedProtocolMessageType('ChartMediatorFormulaStorage', (_message.Message,), {
  'DESCRIPTOR' : _CHARTMEDIATORFORMULASTORAGE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.ChartMediatorFormulaStorage)
  })
_sym_db.RegisterMessage(ChartMediatorFormulaStorage)

ChartMediatorArchive = _reflection.GeneratedProtocolMessageType('ChartMediatorArchive', (_message.Message,), {
  'DESCRIPTOR' : _CHARTMEDIATORARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.ChartMediatorArchive)
  })
_sym_db.RegisterMessage(ChartMediatorArchive)

ChartSelectionArchive = _reflection.GeneratedProtocolMessageType('ChartSelectionArchive', (_message.Message,), {
  'DESCRIPTOR' : _CHARTSELECTIONARCHIVE,
  '__module__' : 'TNArchives_pb2'
  # @@protoc_insertion_point(class_scope:TN.ChartSelectionArchive)
  })
_sym_db.RegisterMessage(ChartSelectionArchive)


_UISTATEARCHIVE.fields_by_name['active_sheet_index']._options = None
_UISTATEARCHIVE.fields_by_name['selected_info']._options = None
_UISTATEARCHIVE.fields_by_name['sheet_uistate_dictionary_entry']._options = None
_UISTATEARCHIVE.fields_by_name['table_selection']._options = None
_UISTATEARCHIVE.fields_by_name['editing_sheet_index']._options = None
_UISTATEARCHIVE.fields_by_name['edit_mode_sheet_uistate_dictionary_entry']._options = None
_UISTATEARCHIVE.fields_by_name['table_editing_mode']._options = None
_UISTATEARCHIVE.fields_by_name['form_focused_record_index']._options = None
_UISTATEARCHIVE.fields_by_name['form_focused_field_index']._options = None
_UISTATEARCHIVE.fields_by_name['chart_selection']._options = None
_UISTATEARCHIVE.fields_by_name['sheet_selection']._options = None
_UISTATEARCHIVE.fields_by_name['shows_sidebar']._options = None
_DOCUMENTARCHIVE.fields_by_name['calculation_engine']._options = None
_DOCUMENTARCHIVE.fields_by_name['printer_id']._options = None
_SHEETARCHIVE.fields_by_name['show_repeating_headers']._options = None
_SHEETARCHIVE.fields_by_name['header_storage']._options = None
_SHEETARCHIVE.fields_by_name['footer_storage']._options = None
_CHARTMEDIATORARCHIVE.fields_by_name['is_registered_with_calc_engine']._options = None
_CHARTSELECTIONARCHIVE.fields_by_name['deprecated_super']._options = None
# @@protoc_insertion_point(module_scope)
